import asyncio
import os
from dataclasses import dataclass
from typing import Any, Callable, Dict, Tuple, Type

from l2l import Lane, Mock
from l2l.types import LaneDictType
from textual import on
from textual.app import App
from textual.binding import Binding
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import (
    Button,
    Footer,
    Input,
    Label,
    ListItem,
    ListView,
    Markdown,
    Switch,
    TabbedContent,
    TabPane,
    Tree,
)
from textual.widgets.tree import TreeNode
from textual_slider import Slider

from carabao import form

from ...cfg.secret_cfg import SECRET_CFG
from ...helpers.utils import _str2bool, clean_docstring


@dataclass
class Result:
    lane: Type[Lane]
    name: str
    test_mode: bool
    form: dict[str, Any]
    raw_form: dict[str, str]


class Display(App[Result]):
    BINDINGS = [
        Binding("escape", "exit_app", "Exit", priority=True),
        Binding("enter", "run_lane", "Run", priority=True),
    ]

    CSS_PATH = os.path.join(
        os.path.dirname(__file__),
        "display.tcss",
    )

    lane_list: ListView

    def __compose_lane_list(self):
        try:
            initial_index = self.queue_names.index(
                SECRET_CFG.last_run_queue_name,
            )
        except ValueError:
            initial_index = 0

        self.lane_list = ListView(
            *(
                ListItem(
                    Label(queue_name),
                    id=f"lane-{i}",
                )
                for i, queue_name in enumerate(self.queue_names)
            ),
            id="lanes",
            initial_index=initial_index,
        )

        yield self.lane_list

    def __compose_info(self):
        with Container(id="info-container"):
            yield Label(
                "Name",
                classes="info-label",
            )

            self.name_widget = Label(
                "",
                classes="info-widget",
            )

            yield self.name_widget
            yield Label(
                "Queue Names",
                classes="info-label",
            )

            self.queue_names_widget = Label(
                "",
                classes="info-widget",
            )

            yield self.queue_names_widget
            yield Label(
                "Documentation",
                classes="info-label",
            )

            self.docstring_widget = Markdown(
                "",
                id="docstring",
                classes="info-widget",
            )

            yield self.docstring_widget
            yield Label(
                "Process Tree",
                classes="info-label",
            )

            self.sub_lanes_widget = Tree("")

            yield self.sub_lanes_widget

    def __compose_navi(self):
        yield Button.success(
            "\\[Enter] Run",
            id="run",
        )

        with Horizontal(
            classes="switch",
        ):
            self.test_mode = Switch(
                SECRET_CFG.test_mode,
            )

            yield self.test_mode
            yield Label("🧪 Test Mode")

        yield Button.error(
            "\\[Esc] Exit",
            id="exit",
        )

    def __compose_form(self):
        with Container(id="form-container"):
            yield Label()

    def compose(self):
        self.forms: Dict[str, Dict[str, Tuple[str, Callable]]] = {}
        self.lanes = {
            lane.first_name(): (
                lane,
                sorted(
                    form._get_fields(lane),
                    key=lambda field: field.name,
                ),
            )
            for lane in Lane.available_lanes()
            if lane.primary() and not lane.passive()
        }
        self.queue_names = sorted(self.lanes.keys())

        if not self.queue_names:
            raise Exception("No lanes found!")

        yield Footer()

        with Vertical():
            with Horizontal():
                yield from self.__compose_lane_list()

                with TabbedContent():
                    with TabPane("Info"):
                        yield from self.__compose_info()

                    with TabPane("Form"):
                        yield from self.__compose_form()

            with Horizontal(id="navi-container"):
                yield from self.__compose_navi()

        if (
            self.queue_names
            and self.lane_list.index is not None
            and self.lane_list.index < len(self.queue_names)
        ):
            lane_name = self.queue_names[self.lane_list.index]
            self.__update_info(lane_name)

            asyncio.run(self.__update_form(lane_name))

    def __update_info(self, lane_name: str):
        """
        Update the docstring widget with the selected lane's docstring.
        """
        lane = self.lanes[lane_name][0]

        self.docstring_widget.update(
            clean_docstring(lane.__doc__)
            if lane.__doc__
            else "No documentation available."
        )

        self.name_widget.update(lane.__name__)

        self.queue_names_widget.update(
            ", ".join(lane.name()),
        )

        self.sub_lanes_widget.root.expand_all()

        self.sub_lanes_widget.root.allow_expand = False

        # Build a tree representation of sub-lanes

        self.sub_lanes_widget.clear()

        self.sub_lanes_widget.root.set_label(
            lane.__name__,
        )
        self.build_lane_tree(
            lane.get_lanes(),
            self.sub_lanes_widget.root,
        )

    async def __update_form(
        self,
        lane_name: str,
    ):
        """
        Update the form with the selected lane's fields.
        """
        form_container = self.query_one("#form-container")

        await form_container.remove_children()

        fields = self.lanes[lane_name][1]

        if not fields:
            form_container.mount(
                Label(
                    "Not available. "
                    "You can create one by adding a Form class inside your lane.",
                )
            )

            form_container.mount(
                Markdown(
                    """
```py
class MyLane(Lane):
    class Form:
        example_string: str = "Hello World!"
        example_integer: int = 1
        example_float: float = 1.2
        example_boolean: bool = True

    ...
```
                    """,
                )
            )
            return

        if lane_name not in self.forms:
            _form = SECRET_CFG.get_form(lane_name)
            self.forms[lane_name] = {
                field.name: (
                    (
                        _form[field.name.lower()]
                        if field.name.lower() in _form
                        else str(field.default)
                        if field.default is not None
                        else "",
                        field.cast,
                    )
                )
                for field in fields
            }

        for field in fields:
            value, _ = self.forms[lane_name][field.name]

            form_container.mount(Label(field.name))

            name = f"form-{lane_name}-{field.name}"

            if field.raw_cast is bool:
                form_container.mount(
                    Switch(
                        _str2bool(value),
                        name=name,
                        classes="form-switch",
                    )
                )

            elif field.min_value is not None and field.max_value is not None:
                try:
                    value = int(value)
                except Exception:
                    value = None

                h = Horizontal(
                    classes="slider-container",
                )

                form_container.mount(h)

                slider = Slider(
                    value=value,
                    name=name,
                    min=field.min_value,
                    max=field.max_value,
                    step=field.step,
                )

                h.mount(slider)

                h.mount(
                    Label(
                        str(slider.value),
                        classes=f"{name}-value slider-label",
                    )
                )

            else:
                form_container.mount(
                    Input(
                        value,
                        name=name,
                        type="integer"
                        if field.raw_cast is int
                        else "number"
                        if field.raw_cast is float
                        else "text",
                    )
                )

    def build_lane_node(
        self,
        node: TreeNode,
        name: str,
        priority_number: int,
    ):
        return node.add(
            f"{name + ' ' if name else ''}[dim]{priority_number}[/dim]",
            expand=True,
            allow_expand=False,
        )

    def build_lane_tree(
        self,
        sub_lanes: LaneDictType,
        node: TreeNode,
    ):
        if not sub_lanes:
            return

        for priority_number, sub_lane in sorted(
            (
                (
                    priority_number,
                    sub_lane,
                )
                for priority_number, sub_lane in sub_lanes.items()
                if sub_lane is not None
            ),
            key=lambda x: x[0],
        ):
            if isinstance(sub_lane, str):
                self.build_lane_node(
                    node,
                    sub_lane,
                    priority_number,
                )

            elif isinstance(sub_lane, type):
                self.build_lane_tree(
                    sub_lane.get_lanes(),
                    self.build_lane_node(
                        node,
                        sub_lane.__name__,
                        priority_number,
                    ),
                )

            elif isinstance(sub_lane, dict):
                self.build_lane_tree(
                    sub_lane,
                    self.build_lane_node(
                        node,
                        "",
                        priority_number,
                    ),
                )

            elif isinstance(sub_lane, Mock):
                self.build_lane_tree(
                    sub_lane.lanes,
                    self.build_lane_node(
                        node,
                        "",
                        priority_number,
                    ),
                )
                continue

    def action_exit_app(self):
        """Exit the application."""
        self.exit(None)

    def action_run_lane(self):
        """Run the selected lane."""
        self.on_run()

    @on(Switch.Changed)
    def on_switch_changed(self, event: Switch.Changed):
        name = event.switch.name

        if name is None:
            return

        _, name, field = name.split("-")
        _, cast = self.forms[name][field]
        self.forms[name][field] = str(event.switch.value), cast

    @on(Input.Changed)
    def on_input_changed(self, event: Input.Changed):
        name = event.input.name

        if name is None:
            return

        _, name, field = name.split("-")
        _, cast = self.forms[name][field]
        self.forms[name][field] = event.input.value, cast

    @on(Slider.Changed)
    def on_slider_changed(self, event: Slider.Changed):
        name = event.slider.name

        if name is None:
            return

        self.query_one(
            f".{name}-value",
            Label,
        ).update(str(event.slider.value))

        _, name, field = name.split("-")
        _, cast = self.forms[name][field]
        self.forms[name][field] = str(event.slider.value), cast

    @on(Button.Pressed, "#exit")
    def on_exit(self):
        self.exit(None)

    @on(Button.Pressed, "#run")
    def on_run(self):
        if self.lane_list.index is None:
            return

        if self.lane_list.index >= len(self.queue_names):
            return

        name = self.queue_names[self.lane_list.index]
        _form = self.forms[name] if name in self.forms else {}

        self.exit(
            Result(
                lane=self.lanes[name][0],
                name=name,
                test_mode=self.test_mode.value,
                form={name: field[1](field[0]) for name, field in _form.items()},
                raw_form={name: field[0] for name, field in _form.items()},
            ),
        )

    async def __update(self, list_view: ListView):
        if list_view.id != "lanes":
            return

        if list_view.index is None:
            return

        if list_view.index >= len(self.queue_names):
            return

        lane_name = self.queue_names[list_view.index]

        self.__update_info(lane_name)
        await self.__update_form(lane_name)

    @on(ListView.Selected)
    async def on_list_view_selected(self, event: ListView.Selected):
        await self.__update(event.list_view)

    @on(ListView.Highlighted)
    async def on_list_view_highlighted(self, event: ListView.Highlighted):
        await self.__update(event.list_view)



import os
import re
import sys

import typer
from typing_extensions import Annotated

from carabao import form

from ..cfg.secret_cfg import SECRET_CFG
from ..core import Core
from ..helpers.prompter import Prompter
from ..settings import Settings
from . import cmd_dev, cmd_new, init_prompter

app = typer.Typer()


@app.command(
    help="Run the pipeline in development mode.",
)
def dev(
    name: Annotated[
        str,
        typer.Argument(
            help="The name of the lane to run.",
            is_eager=False,
        ),
    ] = "",
    test_mode: Annotated[
        bool,
        typer.Option(
            "--test-mode",
            "-t",
            help="Run the pipeline in testing mode.",
        ),
    ] = None,  # type: ignore
):
    """
    Run the pipeline in development mode.

    If a lane name is provided, runs that specific lane.
    Otherwise, displays a UI to select a lane to run.

    Args:
        name: The name of the lane to run.
    """
    sys.path.insert(0, os.getcwd())

    cfg_test_mode = test_mode if test_mode is not None else SECRET_CFG.test_mode

    if name.strip() != "":
        Core.start(
            name=name,
            dev_mode=True,
            test_mode=cfg_test_mode,
        )
        return

    Core.initialize(
        name=name,
        dev_mode=True,
        test_mode=cfg_test_mode,
    )

    Core.load_lanes(Settings.get())

    # Draw the display.

    result = cmd_dev.Display().run()

    if result is None:
        return

    SECRET_CFG.write(
        section=SECRET_CFG.LAST_RUN,
        key=SECRET_CFG.QUEUE_NAME,
        value=result.name,
    )

    SECRET_CFG.write(
        section=SECRET_CFG.TEST_MODE,
        key=SECRET_CFG.TEST_MODE,
        value=str(result.test_mode),
    )

    for key, value in result.raw_form.items():
        SECRET_CFG.write(
            section=f"{result.name}{SECRET_CFG.FORM}",
            key=key,
            value=str(value),
        )

    SECRET_CFG.save()

    _form = form._get_form(result.lane)

    if _form is not None:
        for key, value in result.form.items():
            setattr(_form, key, value)
            setattr(form.F, key, value)

    # Run the program again.

    Core.start(
        name=result.name,
        dev_mode=True,
        test_mode=result.test_mode,
    )


@app.command(
    help="Run the pipeline in production mode.",
)
def run(
    name: Annotated[
        str,
        typer.Argument(
            help="The name of the lane to run.",
            is_eager=False,
        ),
    ] = "",
):
    """
    Run the pipeline in production mode.

    This starts the Core with the default settings suitable for production.
    """
    sys.path.insert(0, os.getcwd())
    Core.start(
        name=name if name else None,
        dev_mode=False,
    )


@app.command(
    help="Initialize the project.",
)
def init(
    skip: Annotated[
        bool,
        typer.Option(
            "--skip",
            "-s",
            help="Skip all prompts.",
        ),
    ] = False,
):
    """
    Initialize a new carabao project.

    Creates the necessary directory structure and sample files.

    Args:
        skip: Whether to skip all interactive prompts.
    """
    prompter = Prompter()

    prompter.set("skip", skip)
    prompter.set("root_path", os.path.dirname(__file__))

    prompter.add(
        "should_continue",
        init_prompter.ShouldContinue(),
    )

    prompter.add(
        "use_src",
        init_prompter.UseSrc(),
    )

    prompter.add(
        "lane_directory",
        init_prompter.LaneDirectory(),
    )

    prompter.add(
        "new_starter_lane",
        init_prompter.NewStarterLane(),
    )

    prompter.add(
        "new_settings",
        init_prompter.NewSettings(),
    )

    prompter.add(
        "new_cfg",
        init_prompter.NewCfg(),
    )

    prompter.add(
        "new_env",
        init_prompter.NewEnv(),
    )

    prompter.add(
        "update_gitignore",
        init_prompter.UpdateGitIgnore(),
    )

    prompter.query()
    prompter.do()

    typer.echo(
        typer.style(
            "Carabao initialized.",
            fg=typer.colors.GREEN,
        )
    )


@app.command(
    help="Create a new lane.",
)
def new(
    name: Annotated[
        str,
        typer.Argument(help="The name of the lane to create."),
    ] = "",
):
    """
    Create a new lane with the given name.

    This command guides the user through creating a new lane by:
    1. Checking for valid lane directories from settings
    2. Prompting for lane name and directory if not provided
    3. Converting the lane name to appropriate filename and class name formats
    4. Creating the lane file with template content

    If no lane directories are configured, the command will display an error.
    """
    sys.path.insert(0, os.getcwd())

    lane_directories = [
        *map(
            lambda x: x.replace(".", "/"),
            Settings.get().value_of("LANE_DIRECTORIES"),
        ),
    ]

    if not lane_directories:
        typer.secho(
            "No lane directories found!",
            fg=typer.colors.RED,
        )
        return
        # raise Exception("Lane directory not found!")

    display = cmd_new.Display()
    display.default_lane_name = name.strip() or "MyLane"
    display.default_lane_directory = lane_directories[0]

    result: cmd_new.Item = display.run()  # type: ignore

    if not result:
        return

    lane_directories = [result.lane_directory]
    filename = re.sub(
        r"(?<=[a-z])(?=[A-Z0-9])|(?<=[A-Z0-9])(?=[A-Z][a-z])|(?<=[A-Za-z])(?=\d)",
        "_",
        result.lane_name,
    ).lower()
    class_name = "".join(word.capitalize() for word in filename.split("_"))

    for lane_directory in lane_directories:
        if not os.path.exists(lane_directory):
            os.makedirs(lane_directory)

        lane_filepath = os.path.join(
            lane_directory,
            f"{filename}.py",
        )

        if os.path.exists(lane_filepath):
            continue

        with open(lane_filepath, "w") as f:
            f.write(result.content)

        typer.echo(
            typer.style(
                f"Lane '{class_name}' created successfully!",
                fg=typer.colors.GREEN,
            )
        )
        return

    typer.secho(
        f"Lane '{class_name}' already exists!",
        fg=typer.colors.RED,
    )
    # raise Exception(f"Lane '{class_name}' already exists!")



from .base_cfg import BaseCFG


class SecretCFG(BaseCFG):
    LAST_RUN = "last_run"
    QUEUE_NAME = "queue_name"
    TEST_MODE = "test_mode"
    FORM = "_form"

    filepath = ".ignore.carabao.cfg"

    @property
    def last_run_queue_name(self):
        section = self.get_section(self.LAST_RUN)

        return section.get(self.QUEUE_NAME)

    @property
    def test_mode(self):
        section = self.get_section(self.TEST_MODE)

        return section.get(self.TEST_MODE) == "True"

    def get_form(self, lane_name: str):
        return dict(
            self.get_section(
                f"{lane_name}{self.FORM}",
            ).items()
        )


SECRET_CFG = SecretCFG()



from typing import Any

from l2l import Lane


class Main(Lane):
    use_filename: bool = True

    @classmethod
    def passive(cls) -> bool:
        return True

    @classmethod
    def primary(cls) -> bool:
        return True

    @classmethod
    def max_run_count(cls) -> int:
        return 1

    @classmethod
    def priority_number(cls) -> float:
        return -100

    @classmethod
    def condition(cls, name: str):
        return True

    def process(self, value: Any):
        print("Hello World!")



import re


def clean_docstring(text: str):
    try:
        lines = text.splitlines()
        indent_len = min(
            len(match.group(0))
            for line in lines
            if line.strip()
            and (
                match := re.match(
                    r"^\s*",
                    line,
                )
            )
        )

        return "\n".join(line[indent_len:] for line in lines).strip()

    except Exception:
        return text.strip()


def _str2bool(value):
    if isinstance(value, bool):
        return value

    if isinstance(value, str):
        return value.lower() in ("true", "1", "yes", "on")

    return bool(value)



import sys
from io import StringIO
from typing import Iterable


class StdOutCatcher(StringIO):
    """
    A class for capturing stdout output while still allowing it to be printed.

    This class intercepts writes to stdout, stores them in an internal buffer,
    and also forwards them to the original stdout.
    """

    def write(self, s: str):
        """
        Write a string to the buffer and to the original stdout.

        Args:
            s: The string to write.

        Returns:
            int: The number of characters written.
        """
        result = super().write(s)

        self.__stdout.write(s)

        return result

    def writelines(self, lines: Iterable[str]):
        """
        Write multiple lines to the buffer and to the original stdout.

        Args:
            lines: An iterable of strings to write.

        Returns:
            None
        """
        result = super().writelines(lines)

        self.__stdout.writelines(lines)

        return result

    def open(self):
        """
        Start capturing stdout by replacing sys.stdout with this instance.

        Stores the original stdout for later restoration.
        """
        self.__stdout = sys.stdout
        sys.stdout = self

    def close(self):
        """
        Stop capturing stdout and restore the original stdout.
        """
        if "__stdout" not in self.__dict__:
            return

        sys.stdout = self.__stdout
        del self.__stdout



from l2l import Lane


class Process1(Lane):
    def process(self, value):
        yield "Hello"
        yield "World"


class Process2(Lane):
    def process(self, value):
        yield value


class Process3(Lane):
    def process(self, value):
        print(value)

        yield value


class Main(Lane):
    use_filename: bool = True

    lanes = {
        1: Process1,
        2: Process2,
        3: Process3,
    }



from .constants import C
from .core import Core
from .form import F, Field, Form
from .lanes import LogToDB, NetworkHealth, PrettyEnv
from .settings import Settings

try:
    from .constants import mongo

except Exception:
    pass

try:
    from .constants import redis

except Exception:
    pass

try:
    from .constants import es

except Exception:
    pass

try:
    from .constants import pg

except Exception:
    pass


def start():
    """
    Starts the Carabao framework.

    This is the main entry point to initialize and run the application.
    """
    Core.start()


__all__ = [
    "C",
    "Core",
    "LogToDB",
    "NetworkHealth",
    "PrettyEnv",
    "Settings",
    "start",
    "mongo",
    "redis",
    "es",
    "pg",
    "Field",
    "F",
    "Form",
]



import os

from textual import on
from textual.app import App
from textual.binding import Binding
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import (
    Button,
    Footer,
    Input,
    Label,
    ListItem,
    ListView,
    Markdown,
    Switch,
)

from carabao.cli.cmd_new.item import Item


class Display(App):
    default_lane_name: str = "MyLane"
    default_lane_directory: str = "lanes"

    BINDINGS = [
        Binding("escape", "exit_app", "Exit", priority=True),
        Binding("enter", "run_lane", "Run", priority=True),
    ]

    CSS_PATH = os.path.join(
        os.path.dirname(__file__),
        "display.tcss",
    )

    SAMPLES_FOLDERPATH = os.path.normpath(
        os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            "../sample",
        )
    )
    TEMPLATES = sorted(
        [
            {
                "name": "Basic Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "basic.py",
                ),
                "description": "A simple lane that processes data sequentially.",
            },
            {
                "name": "Factory Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "factory.py",
                ),
                "description": "A factory pattern implementation for lane processing.",
            },
            {
                "name": "Passive Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "passive.py",
                ),
                "description": "A lane that runs in the background.",
            },
            {
                "name": "Subscriber Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "subscriber.py",
                ),
                "description": "A lane that implements the publisher-subscriber pattern to receive events.",
            },
            {
                "name": "Branching Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "branching.py",
                ),
                "description": "A lane that branches to multiple lanes based on the input.",
            },
            {
                "name": "Batched Subscriber Lane",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "batched_subscriber.py",
                ),
                "description": "A lane that processes data in batches.",
            },
            {
                "name": "Lane with Form",
                "file": os.path.join(
                    SAMPLES_FOLDERPATH,
                    "form.py",
                ),
                "description": "A lane that has a form.",
            },
        ],
        key=lambda x: x["name"],
    )

    def compose(self):
        yield Footer()

        with Vertical():
            with Horizontal():
                # Create ListView with template names
                self.template_list = ListView(
                    *(ListItem(Label(template["name"])) for template in self.TEMPLATES),
                    id="template-list",
                )

                yield self.template_list

                # Vertical container for inputs and info
                with Vertical(id="right-container"):
                    # Text inputs section
                    with Container(id="inputs-container"):
                        yield Label(
                            "Name",
                            classes="input-label",
                        )

                        self.name_input = Input(
                            placeholder=self.default_lane_name,
                            id="name-input",
                        )

                        yield self.name_input

                        yield Label(
                            "Directory",
                            classes="input-label",
                        )

                        self.directory_input = Input(
                            placeholder=self.default_lane_directory,
                        )

                        yield self.directory_input

                        with Horizontal(
                            classes="switch",
                        ):
                            self.use_filename = Switch(
                                True,
                                id="use-filename",
                            )
                            yield self.use_filename
                            yield Label("Use Filename as Name?")

                    # Container for template content
                    with Container(id="info-container"):
                        yield Label(
                            "Description",
                            classes="info-label",
                        )

                        self.description_widget = Label(
                            "",
                            classes="info-widget",
                        )

                        yield self.description_widget
                        yield Label(
                            "Content Preview",
                            classes="info-label",
                        )

                        self.content_widget = Markdown(
                            "",
                            id="content",
                            classes="info-widget",
                        )

                        yield self.content_widget

            # Container for action buttons at bottom
            with Horizontal(id="navi-container"):
                yield Button.success(
                    "\\[Enter] Create",
                    id="select",
                )

                yield Button.error(
                    "\\[Esc] Cancel",
                    id="exit",
                )

        self.update(0)

    def update(self, index: int):
        self.update_item(index)
        self.update_info(index)

    def update_item(self, index: int):
        template = self.TEMPLATES[index]

        with open(template["file"], "r") as f:
            content = f.read()

        lane_name = self.name_input.value or self.default_lane_name

        if not self.use_filename.value:
            content = content.replace(
                "class Main(Lane):",
                f"class {lane_name}(Lane):",
            ).replace(
                "use_filename: bool = True",
                "use_filename: bool = False",
            )

        self.__item = Item(
            lane_name=lane_name,
            lane_directory=self.directory_input.value or self.default_lane_directory,
            use_filename=self.use_filename.value,
            content=content,
        )

    def update_info(self, index: int):
        template = self.TEMPLATES[index]

        self.description_widget.update(template["description"])

        try:
            content = self.__item.content.replace(
                "[",
                "\\[",
            ).replace(
                "]",
                "\\]",
            )

            self.content_widget.update(
                f"```py\n{content}\n```",
            )
        except Exception:
            self.content_widget.update(
                "Could not load template content.",
            )

    def action_exit_app(self):
        self.on_exit()

    def action_run_lane(self):
        self.on_select()

    @on(Input.Changed)
    def on_name_input_changed(self, event: Input.Changed):
        self.update(self.template_list.index or 0)

    @on(Switch.Changed)
    def on_use_filename_changed(self):
        self.update(self.template_list.index or 0)

    @on(Button.Pressed, "#exit")
    def on_exit(self):
        self.exit(None)

    @on(ListView.Highlighted, "#template-list")
    def on_template_selected(self, event: ListView.Selected):
        self.update(event.list_view.index or 0)

    @on(Button.Pressed, "#select")
    def on_select(self):
        self.exit(self.__item)



from l2l import Lane

from carabao.form import F, Field


class Main(Lane):
    class Form:
        value_1: str
        value_2: int
        value_3: float
        value_4: bool
        value_5 = Field(
            cast=int,
            min_value=0,
            max_value=10,
            step=2,
        )

    use_filename: bool = True

    @classmethod
    def primary(cls) -> bool:
        return True

    def process(self, value):
        print("Value 1:", F.value_1)
        print("Value 2:", F.value_2)
        print("Value 3:", F.value_3)
        print("Value 4:", F.value_4)
        print("Value 5:", F.value_5)
